INHERITED_TEXT = "Predefined Pset"
PREDEFINED_PROPERTY_WINDOW_NAME = "Predefined Properties"
DESCRIPTION = "description"
OPTIONAL = "optional"
CURRENT_PR0JECT_PHASE = "current_project_phase"
PROJECT_PHASES = "ProjectPhases"
AGGREGATION_PSET = "AggregationPsetName"
AGGREGATION_ATTRIBUTE = "AggregationAttributeName"
PREDEFINED_PSETS = "PredefinedPropertySets"
PREDEFINED_PSET = "PredefinedPropertySet"
PROPERTY_SET = "PropertySet"
PROPERTY_SETS = "PropertySets"
IDENT_ATTRIBUTE = "ident_attribute"
ATTRIBUTE = "Attribute"
ATTRIBUTES = "Attributes"
OBJECT = "Object"
OBJECTS = "Objects"
AGGREGATE = "Aggregate"
AGGREGATIONS = "Aggregations"
NAME = "name"
IDENTIFIER = "identifer"
PARENT = "parent"
NONE = "None"
DATA_TYPE = "data_type"
VALUE_TYPE = "value_type"
IS_IDENTIFIER = "is_identifier"
CHILD_INHERITS_VALUE = "child_inherits_value"
IS_CONCEPT = "is_concept"
SCRIPT = "Script"
SCRIPTS = "Scripts"
PROJECT = "Project"
VERSION = "version"
AGGREGATES_TO = "aggregates_to"
AUTHOR = "author"
X_POS = "x_pos"
Y_POS = "y_pos"
IS_ROOT = "root"
CONNECTION = "connection"
NODE = "Node"
NODES = "Nodes"
IFC_MAPPINGS = "IfcMappings"
IFC_MAPPING = "IfcMapping"
CUSTOM_ATTRIBUTES = "CustomAttributes"
CUSTOM_ATTRIBUTE = "CustomAttribute"
ABBREVIATION = "abbreviation"
REVIT_MAPPING = "revit_mapping"
VALUE = "Value"
IGNORE_PSET = "IFC"