DEFAULT_NS = "{http://standards.buildingsmart.org/IDS}"
SCHEME_LOCATION_NS = "http://standards.buildingsmart.org/IDS/ids_05.xsd"

# Classes
APPLICABILITY = DEFAULT_NS + 'applicability'
APPLICABILITYTYPE = DEFAULT_NS + 'applicabilityType'
ATTRIBUTE = DEFAULT_NS + 'attribute'
ATTRIBUTETYPE = DEFAULT_NS + 'attributeType'
AUTHOR = DEFAULT_NS + 'author'
CLASSIFICATION = DEFAULT_NS + 'classification'
CLASSIFICATIONTYPE = DEFAULT_NS + 'classificationType'
COPYRIGHT = DEFAULT_NS + 'copyright'
DATE = DEFAULT_NS + 'date'
DESCRIPTION = DEFAULT_NS + 'description'
ENTITY = DEFAULT_NS + 'entity'
ENTITYTYPE = DEFAULT_NS + 'entityType'
IDS = DEFAULT_NS + 'ids'
IDSVALUE = DEFAULT_NS + 'idsValue'
INFO = DEFAULT_NS + 'info'
MATERIAL = DEFAULT_NS + 'material'
MATERIALTYPE = DEFAULT_NS + 'materialType'
MILESTONE = DEFAULT_NS + 'milestone'
NAME = DEFAULT_NS + 'name'
PARTOF = DEFAULT_NS + 'partOf'
PARTOFTYPE = DEFAULT_NS + 'partOfType'
PREDEFINEDTYPE = DEFAULT_NS + 'predefinedType'
PROPERTY = DEFAULT_NS + 'property'
PROPERTYSET = DEFAULT_NS + 'propertySet'
PROPERTYTYPE = DEFAULT_NS + 'propertyType'
PURPOSE = DEFAULT_NS + 'purpose'
REQUIREMENTS = DEFAULT_NS + 'requirements'
REQUIREMENTSTYPE = DEFAULT_NS + 'requirementsType'
SIMPLEVALUE = DEFAULT_NS + 'simpleValue'
SPECIFICATION = DEFAULT_NS + 'specification'
SPECIFICATIONTYPE = DEFAULT_NS + 'specificationType'
SPECIFICATIONS = DEFAULT_NS + 'specifications'
SPECIFICATIONSTYPE = DEFAULT_NS + 'specificationsType'
SYSTEM = DEFAULT_NS + 'system'
TITLE = DEFAULT_NS + 'title'
VALUE = DEFAULT_NS + 'value'
VERSION = DEFAULT_NS + 'version'

# Attributes
ATTR_DATATYPE = 'datatype'
ATTR_DESCRIPTION = 'description'
ATTR_IDENTIFIER = 'identifier'
ATTR_IFCVERSION = 'ifcVersion'
ATTR_INSTRUCTIONS = 'instructions'
ATTR_NAME = 'name'
ATTR_RELATION = 'relation'
ATTR_URI = 'uri'

# Values
VAL_IFC2X3 = 'IFC2X3'
VAL_IFC4 = 'IFC4'
VAL_IFC4X3 = 'IFC4X3'
VAL_IFCRELAGGREGATES = 'IFCRELAGGREGATES'
VAL_IFCRELASSIGNSTOGROUP = 'IFCRELASSIGNSTOGROUP'
VAL_IFCRELCONTAINEDINSPATIALSTRUCTURE = 'IFCRELCONTAINEDINSPATIALSTRUCTURE'
VAL_IFCRELFILLSELEMENT = 'IFCRELFILLSELEMENT'
VAL_IFCRELNESTS = 'IFCRELNESTS'
VAL_IFCRELVOIDSELEMENT = 'IFCRELVOIDSELEMENT'
